/*******************************************************************************  
 *  (c) by Knobloch GmbH                          Version Ulrich Mller 09.05.05
 *         Weedgasse 14
 *         D-55234 Erbes-Bdesheim                                    
 *
 *  Header: umFtLib.h Headerfile for fischertechnik ROBO Interface Library
 *
 *  Version:        0.38
 *  Date:           28. April 2005
 *            
 ******************************************************************************/


#ifndef _FT_LIBRARY_H_
#define _FT_LIBRARY_H_


// --- Communication Area for ft-Device ---
typedef struct _FT_TRANSFER_AREA
{
	// --- Digital Inputs I8 - I1 ..... I32 - I25 (Base+0x00)
    unsigned char E_Main;
    unsigned char E_Sub1;
    unsigned char E_Sub2;
    unsigned char E_Sub3;

    unsigned char rsvd1[8];

	// --- Abstandsensoren (1=detektiert) D2 - D1 (Base+0x0C)
    unsigned char Distance;

    unsigned char rsvd2;

  // --- IR-Receiver input code C - TTTT (Base+0x0E)
    unsigned char IRKeys;

    unsigned char res_1;

	// Analoginputs 8x 16 Bit, Range 0..1023, L:H Format (intel) (Base+0x10)
    unsigned short AX;
    unsigned short AY;
    unsigned short A1;
    unsigned short A2;
    unsigned short AZ;
    unsigned short AV;
    unsigned short D1;
    unsigned short D2;
    unsigned short AXS1;
    unsigned short AXS2;
    unsigned short AXS3;

	// Detection limit for distance sensors ( to generate digital from analog input )
  // Base+0x26..0x27: DS1
    unsigned short DS1;
    unsigned short DS2;

	// Fast counter input (16-Bit, 26pin. Extensionconnector) 
  // Base+0x2A..0x2B: ZE (Master Modul)
    unsigned short ZE;

    unsigned char rsvd3[4];

    // --- 16-Bit Timers (Base+0x30..0x31)
    unsigned short Timer1ms;
    unsigned short Timer10ms;
    unsigned short Timer100ms;
    unsigned short Timer1s;
    unsigned short Timer10s;
    unsigned short Timer1min;

    unsigned short res_bF;
    unsigned char res_bT;
    unsigned char rsvd4[1];

	// --- Outputs of the main module (polarity) Base+0x40: M4B - M1A
    unsigned char M_Main;
    unsigned char M_Sub1;
    unsigned char M_Sub2;
    unsigned char M_Sub3;

    unsigned char rsvd5[4];

    // --- Outputs of the main module ( energy saver mode )
    //     1 = energy saver mode, 0 = operational, PowerUp = 1
    //     Base+0x48: | 0  | 0  | 0  | 0  | M4 | M3 | M2 | M1 |
    unsigned char MES_Main;
    unsigned char MES_Sub1;
    unsigned char MES_Sub2;
    unsigned char MES_Sub3;

    unsigned char rsvd6[1];
    unsigned char reserved_l1;
    unsigned char reserved_l2;
    unsigned char reserved_l3;

	// --- Outputs of the main module (PWM values, range =0..7)
  // Base+0x50: M1A - M4B
    unsigned char MPWM_Main[8];
    unsigned char MPWM_Sub1[8];
    unsigned char MPWM_Sub2[8];
    unsigned char MPWM_Sub3[8];

    unsigned char rsvd7[112];

    // --- Operation mode  Base+0xE0: 0x00=online, 0x01=binary download
    unsigned char Mode;

    // --- Output PWM update (0x01=always, 0x02=once)
    // Base+0xE1: | 0  | 0  | 0  | 0  | 0  | 0  |ONCE|ALWA|
    unsigned char MPWM_Update;

    // --- Base+0xE2 0 = Transfer Area closed 1 = Transfer Area is working
    unsigned char TransferAktiv;

    unsigned char rsvd8[3];

    // Bus connected extension modules Base+0xE6: | 0  | 0  | 0  | 0  | 0  | S3 | S2 | S1 |
    unsigned char BusModules;

    // Slot 1 connected extension module type
    // Base+0xE7: ?
    unsigned char SlotModule1;

    // Slot 2 connected extension module type
    // Base+0xE8: ?
    unsigned char SlotModule2;

    unsigned char rsvd9a[7];


    // --- Change Bytes
    //     1 = EG (Master or Slave 1..3) have changed
    //     The Application must read and change this byte to "0"
    //     Use the Windows InterlockedExchange() Function!
    unsigned char ChangeEg;
    unsigned char ChangeAn;
    unsigned char ChangeIr;

    unsigned char rsvd9b[13];


	// --- Digital inputs as 16 bit values (Base+0x100)
    unsigned short E16_Main[8];
    unsigned short E16_Sub1[8];
    unsigned short E16_Sub2[8];
    unsigned short E16_Sub3[8];
    unsigned short Distance16[2];

    unsigned char rsvd10[12];
 
	// --- IR-Keys as 16 bit values (independent of code) (Base+0x150)
  // M3R M3L M1 M2 M3 Code2 M1BW M1FW M2L M2R Code1
    unsigned short IRKeys16[11];
    unsigned char rsvd11[10];

    unsigned short IRKeys16Code1[11];

    unsigned char rsvd12[10];
 
    unsigned short IRKeys16Code2[11];

    unsigned char rsvd13[10];

    DWORD dwTC;

    unsigned char rsvd14[76];


} FT_TRANSFER_AREA;



                                            // --- Numbers from GetFtDeviceTyp()
#define NO_FT_DEVICE                0       // No ft Device connected
#define FT_AUTO_TYPE                1       // Search for Device
#define FT_INTELLIGENT_IF           10      // FT-Intelligent Interface connect (serial)
#define FT_INTELLIGENT_IF_SLAVE     20      // FT-Intelligent Interface with Extension 
																						// connect (serial)
#define FT_ROBO_IF_IIM              50      // FT-Robo Interface with 
																						// Intelligent-Interface-Modus connect (serial)
#define FT_ROBO_IF_USB              60      // FT-Robo Interface connect with USB-Port
#define FT_ROBO_IF_COM              70      // FT-Robo Interface connect with COM- (serial-) Port
#define FT_ROBO_IF_OVER_RF          80      // FT-Robo Interface connect over RF-Data-Link
#define FT_ROBO_IO_EXTENSION        90      // FT-Robo I/O-Extension
#define FT_ROBO_RF_DATA_LINK        110     // FT-Robo RF Data Link
#define FT_MAX_TYP_NUMBER           999     // Max Typ Number

#define FT_ANALOG_CYCLE             5       // Standard: 5*Read Digital Inputs, then EX, EY
                                            // Only for old Intelligent Interface 
                                            // { for OpenFtCommDevice() }


#define PORT_NONE                   0
#define PORT_COM                    1       // Standard: COM1
#define PORT_COM1                   1
#define PORT_COM2                   2
#define PORT_COM3                   3
#define PORT_COM4                   4
#define PORT_USB                    50

                                            // --- Moduletypes for Robo Interface
#define MODULE_NOT_INSTALLED        0       // Not installed
#define MODULE_RF1                  1       // RF Data Link Module is installed


#define LEN_STRG1                   30      // Length of SN / FW Strings
#define LEN_STRG2                   255     // Length of Name-Strigs
#define ANZ_DISTANCE_TYPES          2       // Number of supported Distance-Modes
#define LEN_DIST_TYPE_NAMES         30      // Lenght of Mode-Names
#define LEN_DIST_VAR_NAMES          20      // Lenght of Variable-Names
#define NAME_LEN                    85      // Lenght of Prog-Names


#define MEM_FLASH1                  0       // Use Memory FLASH-1 Block
#define MEM_FLASH2                  1       // Use Memory FLASH-2 Block
#define MEM_RAM                     2       // Use Memory RAM Block
#define MEM_NO_ACCESS               -1      // No Memory Select (for SetFtProgramActiv() )
#define DOWNLOAD_PROG_NAMELEN       80      // Max. 80 characters/bytes for the programname
                                            // Values for "dwParameter" from 
																						// DownloadFtProgram() 
#define PROG_TYP_NORMAL             1       // Program can bei Start with Software and Switch
#define PROG_TYP_AUTOSTART          2       // Program is startable at Power-On


// -------------------------------------- UM ---------------------------------------------------


    typedef void * FT_HANDLE;
    typedef void (*DATA_UPDATE_CALLBACK)(void* Context);


    typedef struct _NOTIFICATION_EVENTS 
    {
        DATA_UPDATE_CALLBACK NotificationCallback;
        void* Context;
        HANDLE hEvent;
        HWND    WindowsHandle;   
        DWORD   Message;         
    } NOTIFICATION_EVENTS;

    typedef struct _FT_SERIAL_NUMBER
    {
        DWORD   dwSn;                
        DWORD   dwSn1;               
        DWORD   dwSn2;               
        DWORD   dwSnAktiv;           
    } FT_SERIAL_NUMBER;

    typedef struct _FT_DISTANCE 
    {
        unsigned char   aDistModeNames[LEN_DIST_TYPE_NAMES];  
        DWORD           dwEnableDisVal1;                    
        unsigned char   aDisVal1Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal1;                          
        DWORD           dwEnableDisVal2;                    
        unsigned char   aDisVal2Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal2;                          
        DWORD           dwEnableDisVal3;                    
        unsigned char   aDisVal3Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal3;                          
        DWORD           dwEnableDisVal4;                    
        unsigned char   aDisVal4Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal4;                          
        DWORD           dwEnableDisVal5;                    
        unsigned char   aDisVal5Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal5;                          
        DWORD           dwEnableDisVal6;                    
        unsigned char   aDisVal6Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal6;                          
        DWORD           dwEnableDisVal7;                    
        unsigned char   aDisVal7Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal7;                          
        DWORD           dwEnableDisVal8;                    
        unsigned char   aDisVal8Name[LEN_DIST_VAR_NAMES];   
        DWORD           dwDisVal8;                          
    } FT_DISTANCE;



    typedef union _FT_SETTING
    {
			struct {
				DWORD   dwIfTyp; 
				DWORD   dwSn;   
				DWORD   dwSn1;  
				DWORD   dwSn2;
        DWORD   dwSnAktiv; 
        
				DWORD   dwFw;

        unsigned char aSnStrg[LEN_STRG1];
        unsigned char aSn1Strg[LEN_STRG1];  
        unsigned char aSn2Strg[LEN_STRG1];  
        unsigned char aFwStrg[LEN_STRG1];   
        unsigned char aSNaStrg[LEN_STRG2];  
        unsigned char aLNaStrg[LEN_STRG2];  

        DWORD   dwFwRoboIfVerAtLibIsMade; 
        DWORD   dwFwIoExtVerAtLibIsMade;  
        DWORD   dwFwRfDataVerAtLibIsMade; 
        unsigned char aFwRoboIfVerAtLibIsMadeStrg[LEN_STRG1];
        unsigned char aFwIoExtVerAtLibIsMadeStrg[LEN_STRG1]; 
        unsigned char aFwRfDataVerAtLibIsMadeStrg[LEN_STRG1];
                                            
        DWORD   dwAnzExtModules;            
                                                
        DWORD   dwOutputAutoOff[4];         
        DWORD   dwOutputPwmType;            

        DWORD   dwAnzDistanceModes;         
        DWORD   dwDistanceMode;             
        FT_DISTANCE     sDistance[ANZ_DISTANCE_TYPES];  

				DWORD           dwProg[3];
        unsigned char   aProgNames[3][NAME_LEN];  
        DWORD           dwF1AutoStart;            
                     
        DWORD           dwMod1Typ;
        DWORD           dwMod2Typ;

        unsigned char   aMod1Strg[LEN_STRG1];   
        unsigned char   aMod2Strg[LEN_STRG1];   
                                                
            DWORD   dwRfOn;                     
            DWORD   dwRfChannel;                 
            DWORD   dwRfSubChannel;             
            DWORD   dwRfMessage;                
            DWORD   dwRfWrite;                   
            DWORD   dwRfFw;                     
            unsigned char aRfFwStrg[LEN_STRG1];  
        } sIf;
    } FT_SETTING;

                                                // ****** Function Prototypes  ******
    DWORD __stdcall     GetLibVersion(void);
    
		DWORD __stdcall     InitFtLib(void);
    DWORD __stdcall     CloseFtLib(void);
    DWORD __stdcall     IsFtLibInit(void);
    
		DWORD __stdcall     InitFtUsbDeviceList(void);
    UINT  __stdcall     GetAnzFtUsbDevice(void);
    FT_HANDLE __stdcall GetFtUsbDeviceHandle(UCHAR ucDevNr);
    FT_HANDLE __stdcall GetFtUsbDeviceHandleSerialNr(DWORD dwSN, DWORD dwTyp);
    
		DWORD __stdcall     OpenFtUsbDevice(FT_HANDLE hFt);
    FT_HANDLE __stdcall OpenFtCommDevice(DWORD dwPort, DWORD dwTyp, 
			                                   DWORD dwZyklus, DWORD *pdwError);
    DWORD __stdcall     CloseAllFtDevices(void);
    DWORD __stdcall     CloseFtDevice(FT_HANDLE hFt);
    
		DWORD __stdcall     GetFtDeviceTyp(FT_HANDLE hFt);
    LPCSTR __stdcall    GetFtSerialNrStrg(FT_HANDLE hFt);
    DWORD  __stdcall    GetFtSerialNr(FT_HANDLE hFt);
    LPCSTR __stdcall    GetFtFirmwareStrg(FT_HANDLE hFt);
    DWORD __stdcall     GetFtFirmware(FT_HANDLE hFt);
    LPCSTR __stdcall    GetFtManufacturerStrg(FT_HANDLE hFt);
    LPCSTR __stdcall    GetFtShortNameStrg(FT_HANDLE hFt);
    LPCSTR __stdcall    GetFtLongNameStrg(FT_HANDLE hFt);
    LPCSTR __stdcall    GetFtLibErrorString(DWORD dwErrorCode, DWORD dwTyp);
    DWORD __stdcall     GetFtDeviceSetting(FT_HANDLE hFt, FT_SETTING *pSet);
    
		DWORD __stdcall     SetFtDeviceSetting(FT_HANDLE hFt, FT_SETTING *pSet);
    DWORD __stdcall     SetDistanceSensorMode(FT_HANDLE hFt, DWORD dwMode, DWORD dwTol1, 
			                                        DWORD dwTol2, DWORD dwSchwell1, DWORD dwSchwell2, 
																							DWORD dwRepeat1, DWORD dwRepeat2);
    
		DWORD __stdcall     StartFtTransferArea(FT_HANDLE hFt, NOTIFICATION_EVENTS* sNEvent );
    DWORD __stdcall     StopFtTransferArea(FT_HANDLE hFt);
    
		FT_TRANSFER_AREA* __stdcall GetFtTransferAreaAddress(FT_HANDLE hFt);
    DWORD __stdcall     IsFtTransferActiv(FT_HANDLE hFt);
    DWORD __stdcall     ResetFtTransfer (FT_HANDLE hFt);    
    
		DWORD __stdcall     GetFtMemoryLayout(FT_HANDLE hFt, BYTE * pbArray, DWORD dwSize);
    DWORD __stdcall     DownloadFtProgram(FT_HANDLE hFt, DWORD dwMemBlock, BYTE* pbArray, 
			                                    DWORD dwSize, DWORD dwParameter, 
																					BYTE *pbName, DWORD dwNameLen);
    DWORD __stdcall     StartFtProgram(FT_HANDLE hFt, DWORD dwMemBlock);
    DWORD __stdcall     StopFtProgram(FT_HANDLE hFt);
    DWORD __stdcall     DeleteFtProgram(FT_HANDLE hFt, DWORD dwMemBlock);
    DWORD __stdcall     SetFtProgramActiv(FT_HANDLE hFt, DWORD dwMemBlock);
    DWORD __stdcall     GetFtProgramName(FT_HANDLE hFt, DWORD dwMemBlock, 
			                                   DWORD dwSize, LPVOID pName);
    DWORD __stdcall     WriteFtMemoryData(FT_HANDLE hFt, DWORD dwData, DWORD dwAddress);
    DWORD __stdcall     GetFtMemoryData(FT_HANDLE hFt, BYTE * pbArray, 
			                                  DWORD dwSize, DWORD dwAddress);

//
// FTLIB Error Codes
//
    #define FTLIB_ERR_SUCCESS                              0x00000000L  
    #define FTLIB_ERR_CRC                                  0xE0000001L  
    #define FTLIB_ERR_BTSTUFF                              0xE0000002L  
    #define FTLIB_ERR_DATA_TOGGLE_MISMATCH                 0xE0000003L  
    #define FTLIB_ERR_STALL_PID                            0xE0000004L  
    #define FTLIB_ERR_DEV_NOT_RESPONDING                   0xE0000005L  
    #define FTLIB_ERR_PID_CHECK_FAILURE                    0xE0000006L  
    #define FTLIB_ERR_UNEXPECTED_PID                       0xE0000007L  
    #define FTLIB_ERR_DATA_OVERRUN                         0xE0000008L  
    #define FTLIB_ERR_DATA_UNDERRUN                        0xE0000009L  
    #define FTLIB_ERR_RESERVED1                            0xE000000AL  
    #define FTLIB_ERR_RESERVED2                            0xE000000BL  
    #define FTLIB_ERR_BUFFER_OVERRUN                       0xE000000CL  
    #define FTLIB_ERR_BUFFER_UNDERRUN                      0xE000000DL  
    #define FTLIB_ERR_NOT_ACCESSED                         0xE000000FL  
    #define FTLIB_ERR_FIFO                                 0xE0000010L  
    #define FTLIB_ERR_XACT_ERROR                           0xE0000011L  
    #define FTLIB_ERR_BABBLE_DETECTED                      0xE0000012L  
    #define FTLIB_ERR_DATA_BUFFER_ERROR                    0xE0000013L  
                                                    
    #define FTLIB_ERR_ENDPOINT_HALTED                      0xE0000030L  
    #define FTLIB_ERR_NO_MEMORY                            0xE0000100L  
    #define FTLIB_ERR_INVALID_URB_FUNCTION                 0xE0000200L  
    #define FTLIB_ERR_INVALID_PARAMETER                    0xE0000300L  
    #define FTLIB_ERR_ERROR_BUSY                           0xE0000400L  
    #define FTLIB_ERR_INVALID_PIPE_HANDLE                  0xE0000600L  
    #define FTLIB_ERR_NO_BANDWIDTH                         0xE0000700L  
    #define FTLIB_ERR_INTERNAL_HC_ERROR                    0xE0000800L  
    #define FTLIB_ERR_ERROR_SHORT_TRANSFER                 0xE0000900L  
    #define FTLIB_ERR_BAD_START_FRAME                      0xE0000A00L  
    #define FTLIB_ERR_ISOCH_REQUEST_FAILED                 0xE0000B00L  
    #define FTLIB_ERR_FRAME_CONTROL_OWNED                  0xE0000C00L  
    #define FTLIB_ERR_FRAME_CONTROL_NOT_OWNED              0xE0000D00L  
    #define FTLIB_ERR_NOT_SUPPORTED                        0xE0000E00L  
    #define FTLIB_ERR_INVALID_CONFIGURATION_DESCRIPTOR     0xE0000F00L  

    #define FTLIB_ERR_INSUFFICIENT_RESOURCES               0xE8001000L  
    #define FTLIB_ERR_SET_CONFIG_FAILED                    0xE0002000L  
    #define FTLIB_ERR_USBD_BUFFER_TOO_SMALL                0xE0003000L  
    #define FTLIB_ERR_USBD_INTERFACE_NOT_FOUND             0xE0004000L  
    #define FTLIB_ERR_INVALID_PIPE_FLAGS                   0xE0005000L  
    #define FTLIB_ERR_USBD_TIMEOUT                         0xE0006000L  
    #define FTLIB_ERR_DEVICE_GONE                          0xE0007000L  
    #define FTLIB_ERR_STATUS_NOT_MAPPED                    0xE0008000L  

    #define FTLIB_ERR_CANCELED                             0xE0010000L  
    #define FTLIB_ERR_ISO_NOT_ACCESSED_BY_HW               0xE0020000L  
    #define FTLIB_ERR_ISO_TD_ERROR                         0xE0030000L  
    #define FTLIB_ERR_ISO_NA_LATE_USBPORT                  0xE0040000L  
    #define FTLIB_ERR_ISO_NOT_ACCESSED_LATE                0xE0050000L  
                                                    
    #define FTLIB_ERR_FAILED                               0xE0001000L  
    #define FTLIB_ERR_INVALID_INBUFFER                     0xE0001001L  
    #define FTLIB_ERR_INVALID_OUTBUFFER                    0xE0001002L  
    #define FTLIB_ERR_OUT_OF_MEMORY                        0xE0001003L  
    #define FTLIB_ERR_PENDING_REQUESTS                     0xE0001004L  
    #define FTLIB_ERR_ALREADY_CONFIGURED                   0xE0001005L  
    #define FTLIB_ERR_NOT_CONFIGURED                       0xE0001006L  
    #define FTLIB_ERR_OPEN_PIPES                           0xE0001007L  
    #define FTLIB_ERR_ALREADY_BOUND                        0xE0001008L  
    #define FTLIB_ERR_NOT_BOUND                            0xE0001009L  
    #define FTLIB_ERR_DEVICE_NOT_PRESENT                   0xE000100AL  
    #define FTLIB_ERR_CONTROL_NOT_SUPPORTED                0xE000100BL  
    #define FTLIB_ERR_TIMEOUT                              0xE000100CL  
    #define FTLIB_ERR_INVALID_RECIPIENT                    0xE000100DL  
    #define FTLIB_ERR_INVALID_TYPE                         0xE000100EL  
    #define FTLIB_ERR_INVALID_IOCTL                        0xE000100FL  
    #define FTLIB_ERR_INVALID_DIRECTION                    0xE0001010L  
    #define FTLIB_ERR_TOO_MUCH_ISO_PACKETS                 0xE0001011L  
    #define FTLIB_ERR_POOL_EMPTY                           0xE0001012L  
    #define FTLIB_ERR_PIPE_NOT_FOUND                       0xE0001013L  
    #define FTLIB_ERR_INVALID_ISO_PACKET                   0xE0001014L  
    #define FTLIB_ERR_OUT_OF_ADDRESS_SPACE                 0xE0001015L  
    #define FTLIB_ERR_INTERFACE_NOT_FOUND                  0xE0001016L  
    #define FTLIB_ERR_INVALID_DEVICE_STATE                 0xE0001017L  
    #define FTLIB_ERR_INVALID_PARAM                        0xE0001018L  
    #define FTLIB_ERR_INVALID_POWER_STATE                  0xE000101AL  
    #define FTLIB_ERR_POWER_DOWN                           0xE000101BL  
    #define FTLIB_ERR_VERSION_MISMATCH                     0xE000101CL  
    #define FTLIB_ERR_SET_CONFIGURATION_FAILED             0xE000101DL  

    #define FTLIB_ERR_VID_RESTRICTION                      0xE0001080L  
    #define FTLIB_ERR_ISO_RESTRICTION                      0xE0001081L  
    #define FTLIB_ERR_BULK_RESTRICTION                     0xE0001082L  
    #define FTLIB_ERR_EP0_RESTRICTION                      0xE0001083L  
    #define FTLIB_ERR_PIPE_RESTRICTION                     0xE0001084L  
    #define FTLIB_ERR_PIPE_SIZE_RESTRICTION                0xE0001085L  
    #define FTLIB_ERR_CONTROL_RESTRICTION                  0xE0001086L  
    #define FTLIB_ERR_INTERRUPT_RESTRICTION                0xE0001087L  
                                                    
    #define FTLIB_ERR_DEVICE_NOT_FOUND                     0xE0001100L
    #define FTLIB_ERR_SOME_DEVICES_ARE_OPEN                0xE0001101L
    #define FTLIB_ERR_DEVICE_IS_OPEN                       0xE0001102L
    #define FTLIB_ERR_DEVICE_NOT_OPEN                      0xE0001103L
    #define FTLIB_ERR_NO_SUCH_DEVICE_INSTANCE              0xE0001104L
    #define FTLIB_ERR_INVALID_FUNCTION_PARAM               0xE0001105L
    #define FTLIB_ERR_DESTROY_DEVICE_LIST                  0xE0001106L
    #define FTLIB_ERR_USB_NOT_SUPPORTET_FROM_OS            0xE0001107L
    #define FTLIB_ERR_CLOSE_DEVICE                         0xE0001108L

    #define FTLIB_ERR_DEVICE_NOT_SUPPORTET                 0xE0001281L  
    #define FTLIB_ERR_WRONG_DEVICE_NUMBER                  0xE0001282L  
    #define FTLIB_ERR_UNKNOWN_DEVICE_HANDLE                0xE0001283L  
    #define FTLIB_ERR_WRONG_USB_HANDLE                     0xE0001284L  
    #define FTLIB_ERR_WRONG_PATHNAME                       0xE0001285L  
    #define FTLIB_ERR_LIB_IS_INITIALIZED                   0xE0001286L  
    #define FTLIB_ERR_LIB_IS_NOT_INITIALIZED               0xE0001287L  
    #define FTLIB_ERR_SERIAL_NR_UNKNOWN                    0xE0001288L  
    #define FTLIB_ERR_POWER_TOO_LOW                        0xE0001289L  
    #define FTLIB_ERR_WRONG_RETURN_VALUE                   0xE000128AL  
    #define FTLIB_ERR_USB_SUPPORT_IS_NOT_INITIALIZED       0xE000128BL  

    #define FTLIB_ERR_USB_MUT0                             0xE0001290L  
    #define FTLIB_ERR_USB_MUT1                             0xE0001291L  
    #define FTLIB_ERR_USB_MUT2                             0xE0001292L  
    #define FTLIB_ERR_USB_MUT3                             0xE0001293L 
    #define FTLIB_ERR_USB_DEVICE_IS_IN_USE                 0xE0001295L

    #define FTLIB_ERR_THREAD_NOT_STARTABLE                 0xE00012A0L  
    #define FTLIB_ERR_THREAD_NOT_STARTABLE_X1              0xE00012A1L  
    #define FTLIB_ERR_THREAD_NOT_STARTABLE_X2              0xE00012A2L  
    #define FTLIB_ERR_THREAD_NOT_STARTABLE_X3              0xE00012A3L  
    #define FTLIB_ERR_OTHER_THREAD_FOR_SAME_DEVICE_IS_RUN  0xE00012A4L  
    #define FTLIB_ERR_THREAD_IS_RUNNING                    0xE00012A5L  
    #define FTLIB_ERR_THREAD_NOT_RUNNING                   0xE00012A6L  
    #define FTLIB_ERR_THREAD_X1                            0xE00012A7L  
    #define FTLIB_ERR_THREAD_X2                            0xE00012A8L  
    #define FTLIB_ERR_THREAD_WRITE_USB                     0xE00012A9L  
    #define FTLIB_ERR_THREAD_READ_USB                      0xE00012AAL  
    #define FTLIB_ERR_THREAD_IS_NORMAL_STOPPED             0xE00012ABL  
    #define FTLIB_ERR_THREAD_WAIT_TIMEOUT                  0xE00012ACL  
    #define FTLIB_ERR_THREAD_WRITE_COM                     0xE00012ADL  
    #define FTLIB_ERR_THREAD_READ_COM                      0xE00012AEL   

    #define FTLIB_ERR_DOWNLOAD                             0xE00012C0L  
    #define FTLIB_ERR_DOWNLOAD_WRONG_PARAMETER             0xE00012C1L  
    #define FTLIB_ERR_DOWNLOAD_WRONG_MEM_BLOCK             0xE00012C2L  
    #define FTLIB_ERR_DOWNLOAD_DATA_SIZE_IS_NULL           0xE00012C3L  
    #define FTLIB_ERR_DOWNLOAD_DATA_SIZE_TOO_BIG           0xE00012C4L  
    #define FTLIB_ERR_DOWNLOAD_DATA_SIZE                   0xE00012C5L  
    #define FTLIB_ERR_DOWNLOAD_ERASE                       0xE00012C6L  
    #define FTLIB_ERR_DOWNLOAD_F5                          0xE00012C7L  
    #define FTLIB_ERR_DOWNLOAD_PROG_SNF1                   0xE00012C8L  
    #define FTLIB_ERR_DOWNLOAD_PROG_SNF2                   0xE00012C9L  
    #define FTLIB_ERR_DOWNLOAD_BLOCK                       0xE00012CAL  
    #define FTLIB_ERR_DOWNLOAD_BLOCK_NR                    0xE00012CBL  
    #define FTLIB_ERR_DOWNLOAD_CRC                         0xE00012CDL  

    #define FTLIB_ERR_IF_NO_PROGRAM                        0xE00012E0L  
    #define FTLIB_ERR_IF_PROGRAM_IS_RUNNING                0xE00012E1L  
    #define FTLIB_ERR_IF_NO_PROGRAM_IS_RUNNING             0xE00012E2L  
    #define FTLIB_ERR_IF_WRONG_PROG_NUMBER                 0xE00012E3L  
    #define FTLIB_ERR_IF_BUFFER_TOO_SMALL                  0xE00012E4L  
    #define FTLIB_ERR_IF_MEM_ADDRESS                       0xE00012E5L  
    #define FTLIB_ERR_IF_MEM_VERIFY                        0xE00012E6L  
    #define FTLIB_ERR_IF_NOT_SUPPORTET_MODUS               0xE00012E7L  


    #define FTLIB_INFO_PROGRAM_0_IS_RUNNING                0xE0001600L  
    #define FTLIB_INFO_PROGRAM_1_IS_RUNNING                0xE0001601L  
    #define FTLIB_INFO_PROGRAM_2_IS_RUNNING                0xE0001602L  
    #define FTLIB_INFO_ONLINE_MODUS                        0xE0001620L  

    #define FTLIB_FIRMWARE_UPDATE_FILE_WRONG_0             0xE0001800L  
    #define FTLIB_FIRMWARE_UPDATE_CRC                      0xE0001801L  


    #define FTLIB_ERR_PORT_NUMBER                          0xE0001900L  
    #define FTLIB_ERR_WRONG_PORT                           0xE0001901L  
    #define FTLIB_ERR_PORT_NUMBER_IS_NULL                  0xE0001902L  
    #define FTLIB_ERR_TYP                                  0xE0001903L  
    #define FTLIB_ERR_PORT_NOT_FOUND                       0xE0001904L  
    #define FTLIB_ERR_ACCESS_DENIED                        0xE0001905L  
    #define FTLIB_ERR_OPEN_COM                             0xE0001906L  
    #define FTLIB_ERR_CLOSE_COM                            0xE0001907L  
    #define FTLIB_ERR_INIT_COM                             0xE0001908L  
    #define FTLIB_ERR_INIT_COM_TIMEOUT                     0xE0001909L  
    #define FTLIB_ERR_WRITE_COM                            0xE000190AL  
    #define FTLIB_ERR_READ_COM                             0xE000190BL  
    #define FTLIB_ERR_READ_COM_NUMBER                      0xE000190CL  
    #define FTLIB_ERR_READ_COM_NOTHING                     0xE000190DL  

    #define FTLIB_ERR_RF_NOT_INSTALLED                     0xE0001A00L
    #define FTLIB_ERR_RF_WRONG_CHANNEL                     0xE0001A01L
    #define FTLIB_ERR_RF_WRONG_SUB_CHANNEL                 0xE0001A02L
    #define FTLIB_ERR_RF_NO_CONNECTION                     0xE0001A03L
    #define FTLIB_ERR_RF_NO_CONNECTION_A                   0xE0001A04L
    #define FTLIB_ERR_RF_NO_CONNECTION_B                   0xE0001A05L
    #define FTLIB_ERR_RF_NO_CONNECTION_C                   0xE0001A06L
    #define FTLIB_ERR_RF_TIMEOUT_1                         0xE0001A08L
    #define FTLIB_ERR_RF_TIMEOUT_2                         0xE0001A09L
    #define FTLIB_ERR_RF_INTERNAL_1                        0xE0001A0DL

    #define FTLIB_ERR_INTERFACE_00                         0xE0001F00L  
    #define FTLIB_ERR_INTERFACE_01                         0xE0001F01L  
    #define FTLIB_ERR_INTERFACE_02                         0xE0001F02L  
    #define FTLIB_ERR_INTERFACE_03                         0xE0001F03L  
    #define FTLIB_ERR_INTERFACE_04                         0xE0001F04L  
    #define FTLIB_ERR_INTERFACE_05                         0xE0001F05L  
    #define FTLIB_ERR_INTERFACE_06                         0xE0001F06L  
    #define FTLIB_ERR_INTERFACE_07                         0xE0001F07L  
    #define FTLIB_ERR_INTERFACE_08                         0xE0001F08L  
    #define FTLIB_ERR_INTERFACE_09                         0xE0001F09L  
    #define FTLIB_ERR_INTERFACE_0A                         0xE0001F0AL  
    #define FTLIB_ERR_INTERFACE_0B                         0xE0001F0BL  
    #define FTLIB_ERR_INTERFACE_0C                         0xE0001F0CL  
    #define FTLIB_ERR_INTERFACE_0D                         0xE0001F0DL  
    #define FTLIB_ERR_INTERFACE_0E                         0xE0001F0EL  
    #define FTLIB_ERR_INTERFACE_0F                         0xE0001F0FL  

    #define FTLIB_ERR_INTERFACE_D0                         0xE0001FD0L  
    #define FTLIB_ERR_INTERFACE_D1                         0xE0001FD1L  
    #define FTLIB_ERR_INTERFACE_D2                         0xE0001FD2L  
    #define FTLIB_ERR_INTERFACE_D3                         0xE0001FD3L  
    #define FTLIB_ERR_INTERFACE_D4                         0xE0001FD4L  
    #define FTLIB_ERR_INTERFACE_D5                         0xE0001FD5L  
    #define FTLIB_ERR_INTERFACE_D6                         0xE0001FD6L  
    #define FTLIB_ERR_INTERFACE_D7                         0xE0001FD7L  
    #define FTLIB_ERR_INTERFACE_D8                         0xE0001FD8L  
    #define FTLIB_ERR_INTERFACE_D9                         0xE0001FD9L  
    #define FTLIB_ERR_INTERFACE_DA                         0xE0001FDAL  
    #define FTLIB_ERR_INTERFACE_DB                         0xE0001FDBL  
    #define FTLIB_ERR_INTERFACE_DC                         0xE0001FDCL  
    #define FTLIB_ERR_INTERFACE_DD                         0xE0001FDDL  
    #define FTLIB_ERR_INTERFACE_DE                         0xE0001FDEL  
    #define FTLIB_ERR_INTERFACE_DF                         0xE0001FDFL  

    #define FTLIB_ERR_INTERFACE_E0                         0xE0001FE0L  
    #define FTLIB_ERR_INTERFACE_E1                         0xE0001FE1L  
    #define FTLIB_ERR_INTERFACE_E2                         0xE0001FE2L  
    #define FTLIB_ERR_INTERFACE_E3                         0xE0001FE3L  
    #define FTLIB_ERR_INTERFACE_E4                         0xE0001FE4L  
    #define FTLIB_ERR_INTERFACE_E5                         0xE0001FE5L  
    #define FTLIB_ERR_INTERFACE_E6                         0xE0001FE6L  
    #define FTLIB_ERR_INTERFACE_E7                         0xE0001FE7L  
    #define FTLIB_ERR_INTERFACE_E8                         0xE0001FE8L  
    #define FTLIB_ERR_INTERFACE_E9                         0xE0001FE9L  
    #define FTLIB_ERR_INTERFACE_EA                         0xE0001FEAL  
    #define FTLIB_ERR_INTERFACE_EB                         0xE0001FEBL  
    #define FTLIB_ERR_INTERFACE_EC                         0xE0001FECL  
    #define FTLIB_ERR_INTERFACE_ED                         0xE0001FEDL  
    #define FTLIB_ERR_INTERFACE_EE                         0xE0001FEEL  
    #define FTLIB_ERR_INTERFACE_EF                         0xE0001FEFL  

    #define FTLIB_ERR_INTERFACE_F0                         0xE0001FF0L  
    #define FTLIB_ERR_INTERFACE_F1                         0xE0001FF1L  
    #define FTLIB_ERR_INTERFACE_F2                         0xE0001FF2L  
    #define FTLIB_ERR_INTERFACE_F3                         0xE0001FF3L  
    #define FTLIB_ERR_INTERFACE_F4                         0xE0001FF4L  
    #define FTLIB_ERR_INTERFACE_F5                         0xE0001FF5L  
    #define FTLIB_ERR_INTERFACE_F6                         0xE0001FF6L  
    #define FTLIB_ERR_INTERFACE_F7                         0xE0001FF7L  
    #define FTLIB_ERR_INTERFACE_F8                         0xE0001FF8L  
    #define FTLIB_ERR_INTERFACE_F9                         0xE0001FF9L  
    #define FTLIB_ERR_INTERFACE_FA                         0xE0001FFAL  
    #define FTLIB_ERR_INTERFACE_FB                         0xE0001FFBL  
    #define FTLIB_ERR_INTERFACE_FC                         0xE0001FFCL  
    #define FTLIB_ERR_INTERFACE_FD                         0xE0001FFDL  
    #define FTLIB_ERR_INTERFACE_FE                         0xE0001FFEL  
    #define FTLIB_ERR_INTERFACE_FF                         0xE0001FFFL  


#endif



